/*
	Member of opentherm - Modbus - RPi gateway project
	(c) 2015  Faster CZ, spol. s r.o.
*/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MANCHESTER_H
#define __MANCHESTER_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "stm32f0xx.h"


#define MAN_TRESHOLD 		750  // treshold for long/short decision in manchester RX
#define MAN_ERR      		190	 // treshold for error detection in manchester RX
#define MAN_BIT_TIME_MAX   1150	 // treshold for error detection in manchester RX
#define MAN_BIT_TIME_MIN	900	 // treshold for error detection in manchester RX


typedef struct {		
	uint32_t  data;		// receiving data
	uint8_t bit:1;		// bit, that we are gonna write
	uint8_t cnt:6; 		// data counter
	uint16_t prev_capture;  // previous capture time
} MAN_RX_data;

typedef struct {		
	uint32_t  data;		// transmitting data
	uint8_t middle:1; 	// are we in middle of bit?
	uint8_t cnt:7; 		// data counter
} MAN_TX_data;



#define MAN_SEND_0 0
#define MAN_SEND_1 1
#define MAN_SEND_FINISHED 2

#define MAN_RECV_OK 0
#define MAN_RECV_FINISHED 1
#define MAN_RECV_ERROR 2

int manchester_decode(volatile MAN_RX_data* OT, uint16_t capture);
int manchester_encode(volatile MAN_TX_data* TX);

__inline__ void manchester_encode_init(volatile MAN_TX_data* TX, void* data);
__inline__ void manchester_decode_init(volatile MAN_RX_data* RX);


#endif
