/*
	Member of opentherm - Modbus - RPi gateway project
	(c) 2015  Faster CZ, spol. s r.o.
*/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MODBUS_H
#define __MODBUS_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "stm32f0xx.h"


#define swap_word(value) ((value & 0xff00) >> 8) | (value << 8)

#define MB_OP_READ_COIL  		0x01
#define MB_OP_READ_BIT  		0x02
#define MB_OP_READ_HOLD_REG   	0x03
#define MB_OP_READ_REG  	 	0x04
#define MB_OP_WRITE_COIL 		0x05
#define MB_OP_WRITE_REG  		0x06
#define MB_OP_DIAGNOSTIC  		0x08
#define MB_OP_GET_COUNTER 		0x0B
#define MB_OP_WRITE_REGS 		0x10


#define MB_ERR_ILLEGAL_FUNCTION     1
#define MB_ERR_ILLEGAL_DATA_ADDRESS 2
#define MB_ERR_ILLEGAL_DATA_VALUE   3
#define MB_ERR_SLAVE_DEVICE_FAILURE 4
#define MB_ERR_ACKNOWLEDGE 			5
#define MB_ERR_SLAVE_DEVICE_BUSY	6
#define MB_ERR_NOT_ACK				7
#define MB_ERR_MEMORY_PARITY		8

#define MB_DIAG_RETURN_QUERY            0x00
#define MB_DIAG_RESTART_COMM            0x01
#define MB_DIAG_RETURN_DIAG_REG         0x02
#define MB_DIAG_CHANGE_ASCII_DELIM      0x03
#define MB_DIAG_LISTEN_ONLY             0x04
#define MB_DIAG_CLEAR_COUNTERS_AND_DIAG 0x0A
#define MB_DIAG_RET_BUS_MSG_CNT         0x0B
#define MB_DIAG_RET_BUS_COM_ERR_CNT     0x0C
#define MB_DIAG_RET_BUS_EXC_ERR_CNT     0x0D
#define MB_DIAG_RET_SLAVE_MSG_CNT       0x0E
#define MB_DIAG_RET_SLAVE_NORES_CNT     0x0F
#define MB_DIAG_RET_SLAVE_NAK_CNT       0x10
#define MB_DIAG_RET_SLAVE_BUSY_CNT      0x11
#define MB_DIAG_RET_BUS_CHAR_OVRN_CNT   0x12
#define MB_DIAG_RET_CLR_OVRN_CNT_FLAG   0x14



struct mb_counters_struct {
	uint32_t	bus_messages;
	uint32_t	bus_comm_errors;
	uint32_t	exception_errors;
	uint32_t	messages;
	uint32_t	no_response;
	uint32_t	nack;
	uint32_t	char_overrun;

};

struct struct_modbus_cfg {
	uint8_t address			:6; // Modbus address
	uint8_t parity			:1; // USART parity
	uint8_t baud_low		:1; // is baudrate low (9600)?
};



int parity(uint32_t d);
uint16_t CRC_step(volatile uint8_t msg, uint16_t mod_crc);
uint16_t Mod_CRC(volatile uint8_t* data, uint8_t length);



uint8_t mb_check_message(uint8_t length, uint8_t address);
void mb_set_response(uint8_t length);
void mb_set_reg_response(uint16_t value);
void mb_set_coil_response(uint8_t mask);
void mb_writecoil_response(void);
void mb_set_big_response(uint16_t* data_map);

void mb_set_error(uint8_t errorcode);

#define mb_address(data) (data[0])
#define mb_opcode(data)  (data[1])
#define mb_regaddr(data) (((uint16_t) data[2] << 8) + data[3])
#define mb_regcount(data) ((uint16_t)(data[4] << 8) + data[5])
#define mb_value(data) ((uint16_t)(data[4] << 8) + data[5])
#define mb_mvalue(data) ((uint16_t)(data[7] << 8) + data[8])   // for write_regs - 1.value
//#define mb_regaddr(data) ((uint16_t*)&data[2]*)
//#define mb_regcount(data) ((uint16_t*)&data[4]*)

#endif
