/****************************************************************

	Member of opentherm - Modbus - RPi gateway project
	(c) Vclav Steiger, 2015  Faster CZ, spol. s r.o.

****************************************************************/



#include "stm32f0xx.h"
#include "system_stm32f0xx.h"
#include "manchester.h"
#include "modbus.h"


#ifndef USART_CR1_M0
// old name is USART_CR1_M
#define USART_CR1_M0 USART_CR1_M
#endif

#ifndef RCC_APB2ENR_SYSCFGCOMPEN
// old name
#define RCC_APB2ENR_SYSCFGCOMPEN RCC_APB2ENR_SYSCFGEN
#endif

#ifndef COMP1
// old name is COMP
#define COMP1 COMP
#endif


/* ******************************************************************
	Main clock dependencies
		- BRR_HIGH, BRR_LOW = divider for USART clock at 19200 and 9600 Bd
*/

// from 	system_stm32f0xx.c
void SystemCoreClockUpdate (void);

#if 1
// 						HSI+PLL 48MHZ 19200+9600 (value=clock/baud)
//used in interrupts - all volatile
volatile uint8_t BRR_HIGH = 2500;
volatile uint8_t BRR_LOW = 5000;
volatile uint8_t TIMER_PRE = 47;
volatile uint8_t TIME_500_US = 500;
volatile uint8_t TIME_630_US = 630;
volatile uint8_t TIME_20_MS = 20000;
volatile uint8_t TIMER_PRE_LONG = 47999;
volatile uint8_t TIME_LONG_800_MS = 800;
volatile uint8_t TIME_LONG_1_MS = 1;
volatile uint8_t TIME_LONG_1_S = 1000;
#endif

#if 0
// 						HSI 8MHZ 19200+9600
#define BRR_HIGH 416
#define BRR_LOW  833
#define TIMER_PRE 7
#define TIME_500_US 500
#define TIME_630_US 630
#define TIME_20_MS 20000
#define TIMER_PRE_LONG 7999
#define TIME_LONG_1_MS 1
#define TIME_LONG_1_S 1000
#endif

#if 0
// 						HSE+PLL 3.686400 MHZ crystal -> 7.2MHz HSE 	19200+9600
#define BRR_HIGH 384
#define BRR_LOW  768
#define TIMER_PRE 2
#define TIME_500_US 1228        /* 7372800/3 * 0.0005 */
#define TIME_630_US 1547
#define TIME_20_MS  49152
#define TIMER_PRE_LONG 7360
#define TIME_LONG_800_MS 800
#define TIME_LONG_1_MS 1
#define TIME_LONG_1_S 1000
#endif


#if 0
// 						48MHZ 115200+38400 different uart speed
#define BRR_HIGH 417
#define BRR_LOW  1250
#endif


/* ******************************************************************
	Hardware pin dependencies
		- pins and macros
*/

#define LED1 (GPIO_IDR_6)
#define LED2 (GPIO_IDR_7)
#define LED_ON(led) GPIOA->BSRR = led
#define LED_OFF(led) GPIOA->BRR = led
/*
#define LED1 (GPIO_IDR_9)
#define LED2 (GPIO_IDR_8)
#define LED_ON(led) GPIOC->BSRR = led
#define LED_OFF(led) GPIOC->BRR = led
*/

/* PA11 - TX pin to Boiler */
#define BOIL_PIN_LOG1 GPIOA->BRR  = (GPIO_IDR_11) // not a typo, reversed log. levels
#define BOIL_PIN_LOG0 GPIOA->BSRR = (GPIO_IDR_11)

/* PA4 TX pin to Thermostat */
#define THERMO_PIN_LOG0 DAC->DHR12R1 = 616 		/* 0.5V/3.3V * 4095 */
#define THERMO_PIN_LOG1 DAC->DHR12R1 = 2482		/* 2V/3.3V * 4095 */

#define PASSIVE_MONITOR 0x02        // main_mode definition
#define MONITOR         0x01		// main_mode definition
#define CONTROLLER      0x00		// main_mode definition

uint8_t main_mode;			// mode bit array
struct struct_modbus_cfg modbus_cfg;

struct mb_counters_struct mb_counters;
                            //counters for modbus comm

uint16_t data_map[256]; 	// main data map, 128*16bit
uint32_t wset[8];		    // set array - set of opentherm commands invoked by write!


typedef struct {		// message format for OT comm
	uint16_t data_value;
	uint8_t data_id;
	uint8_t msg_type;
} OT_msg;

#define OT_MSGTYPE_MASK    		0x70
#define OT_MSGTYPE_READ    		0x00
#define OT_MSGTYPE_WRITE   		0x10
#define OT_MSGTYPE_INVALID_DATA 0x20
#define OT_MSGTYPE_READ_ACK 	0x40
#define OT_MSGTYPE_WRITE_ACK 	0x50
#define OT_MSGTYPE_DATA_INVALID 0x60
#define OT_MSGTYPE_UNKNOWN_DATAID 0x70

#define OT_STATUS_RX  		0
#define OT_STATUS_RX_CPLT	1
#define OT_STATUS_TX_START	2
#define OT_STATUS_TX  		3
#define OT_STATUS_TX_CPLT	4
#define OT_STATUS_WAITING	5
#define OT_STATUS_WAIT_CPLT	6
#define OT_STATUS_RX_TIME_CPLT	7
#define OT_STATUS_IDLE 		8
#define OT_STATUS_RX_START 	9


struct struct_OT_thermo_comm {		// Opentherm comm status
	uint8_t status;
	uint8_t rx_err		:1; 	// receiving breaked by error
	uint8_t boiler_req  :1; 	// waiting for boiler transaction
};


#define OT_REQ_CYCLING  0
#define OT_REQ_MODBUS   1
#define OT_REQ_THERMO   2

struct struct_OT_boil_comm {		// Opentherm comm status
	uint8_t status;
	uint8_t rx_err		:1; // receiving breaked by error
	uint8_t requestor	:2; // who issued last command (cycling, modbus, thermostat)
};


#define MB_STATUS_RX  		0
#define MB_STATUS_RX_CPLT	1
#define MB_STATUS_TX  		2
#define MB_STATUS_TX_CPLT	3
#define MB_STATUS_WAITING	4
#define MB_STATUS_WAIT_CPLT	5

struct struct_Mod_comm {
	uint8_t status;
	uint8_t boiler_req		:1; // comm request from opentherm boiler
	//uint8_t mod_rx_error	:1; // some error on receiving
};


/* Structures holding communication status of interfaces */
volatile struct struct_OT_thermo_comm OT_thermo_comm;
volatile struct struct_OT_boil_comm   OT_boil_comm;
volatile struct struct_Mod_comm Mod_comm;

uint8_t Modbus_out[255];
volatile uint8_t Modbus_in[255];

volatile uint8_t Modbus_out_length;
volatile uint8_t Modbus_out_index;
volatile uint8_t Modbus_in_length;


volatile MAN_RX_data OT_boiler_in; 	// data struct for boiler manchester decoding
volatile MAN_TX_data OT_boiler_out;	// data struct for boiler manchester encoding
volatile MAN_RX_data OT_thermo_in; 	// data struct for thermostat manchester decoding
volatile MAN_TX_data OT_thermo_out;	// data struct for thermostat manchester encoding

#define OT_boiler_in_data (*(OT_msg*)&OT_boiler_in.data)
#define OT_boiler_out_data (*(OT_msg*)&OT_boiler_out.data)
#define OT_thermo_in_data (*(OT_msg*)&OT_thermo_in.data)
#define OT_thermo_out_data (*(OT_msg*)&OT_thermo_out.data)

OT_msg OT_boiler_mb_data;	    // buffer for OT message from modbus


static volatile uint8_t data_comm_index = 0; // index for data map update in main loop


/* ******************************************************************
	USART Control
		- macros for enable/disble usart rx/tx
		- interrupt handler - TXE, TC, RXNE, RTOF
*/

#define DISABLE_USART1_TX   USART1->CR1 &= ~(USART_CR1_TCIE | USART_CR1_TXEIE | USART_CR1_TE)
#define ENABLE_USART1_TX	USART1->CR1 |=  (USART_CR1_TCIE | USART_CR1_TXEIE | USART_CR1_TE)
// USART_CR1_TE    --  transmission enable
// USART_CR1_TCIE  --  transmission complete interrupt enable (end of transmission)
// USART_CR1_TXEIE --  transmitter empty interrupt update (next char, please...)

#define DISABLE_USART1_RX   USART1->CR1 &= ~(USART_CR1_RE | USART_CR1_RXNEIE | USART_CR1_RTOIE);
#define ENABLE_USART1_RX	USART1->CR1 |= USART_CR1_RE | USART_CR1_RXNEIE | USART_CR1_RTOIE;
// USART_CR1_RE     -- receiver enable
// USART_CR1_RTOIE  -- receiver timeout interrupt enable (end of receive in our case)
// USART_CR1_RXNEIE -- receiver not empty interrupt update (read new char, please...)


/* ***************************************************************************
	Handles timeout between ModBus transactions
*/
void TIM16_IRQHandler(void)
{

	if (((TIM16->SR) & (TIM_SR_UIF)) != 0) {

		TIM16->SR &= ~TIM_SR_UIF;	//clear off the interrupt
		switch (Mod_comm.status) {

		case MB_STATUS_WAITING: {
			// timeout reached;
			Mod_comm.status = MB_STATUS_WAIT_CPLT;
			break;
		}
		}
		TIM16->CR1 &= ~TIM_CR1_CEN;     // turn the timer16 off - give up waiting
		TIM16->DIER &= ~TIM_DIER_UIE;
	}
}

void USART1_IRQHandler(void)
{
	if (((USART1->ISR) & (USART_ISR_TC)) != 0) {
		if ((Mod_comm.status == MB_STATUS_TX) && (Modbus_out_index >= Modbus_out_length)) {
			//transmission complete, back to receive
			USART1->ICR |= USART_ICR_TCCF;
			Mod_comm.status = MB_STATUS_TX_CPLT;
			DISABLE_USART1_TX;	//disable transmission and interrupts
		}
	}

	if (((USART1->ISR) & (USART_ISR_TXE)) != 0) {
		//if transmitter needs new char
		if ((Mod_comm.status == MB_STATUS_TX) && (Modbus_out_index < Modbus_out_length)) {
			USART1->TDR = Modbus_out[Modbus_out_index]; // we provide it
			Modbus_out_index++;
		}
	}

	if (((USART1->ISR) & (USART_ISR_RXNE)) != 0) {
		//if receiver received new char
		if (Mod_comm.status == MB_STATUS_RX) {
			//it is something that we are interested in
			Modbus_in[Modbus_in_length] = USART1->RDR; // we save it
			Modbus_in_length++;
		} else {
			Modbus_in[Modbus_in_length] = USART1->RDR; // read it to clear RXNE flag
		}
	}

	if (((USART1->ISR) & (USART_ISR_RTOF)) != 0) {
		//receive timeout happend if we were receiving, now turn off the receiver and process the data
		USART1->ICR |= USART_ICR_RTOCF;
		if (Mod_comm.status == MB_STATUS_RX) {
			DISABLE_USART1_RX;		 // stop receiving and interrups
			Mod_comm.status = MB_STATUS_RX_CPLT;
		}
		if ((USART1->ISR & (USART_ISR_PE | USART_ISR_ORE | USART_ISR_NE | USART_ISR_FE)) != 0) {
			// parity error, overun, noise error, framing error - clear flags
			USART1->ICR |= (USART_ICR_PECF | USART_ICR_ORECF | USART_ICR_NCF | USART_ICR_FECF);
			Modbus_in_length = 0; 	// this means ERROR
			//Mod_comm.mod_rx_error = 1;
		}
	}
}


/* ***************************************************************************
	Handles intervals between boiler transactions
*/
void TIM15_IRQHandler(void)
{

	if (((TIM15->SR) & (TIM_SR_UIF)) !=0) { //update interrupt on TIM15 - aux timer for manchester comm

		TIM15->SR &= ~TIM_SR_UIF;	//clear off the interrupt
		switch (OT_boil_comm.status) {

			case OT_STATUS_WAITING: {
				// timeout reached;
				OT_boil_comm.status = OT_STATUS_WAIT_CPLT;
				break;
			}
			case OT_STATUS_RX_START:
			case OT_STATUS_RX: {
				// timeout reached by receiving;
				OT_boil_comm.status = OT_STATUS_RX_TIME_CPLT;
				break;
			}
		}
		TIM15->CR1 &= ~TIM_CR1_CEN;     // turn the timer15 off - give up RX
		TIM15->DIER &= ~TIM_DIER_UIE;
	}
}


/****************************************************************************
	Handles Thermostat TX and RX interrupts from TIM3
		- on TX is fired up every 500us
		- on RX is fired on changing rx line connected to comparator COMP1
*/
#if 0
volatile uint8_t stavb = 0;
#endif

void TIM3_IRQHandler(void)
{
	if ((TIM3->SR) & (TIM_SR_UIF)) {
		//update interrupt - Thermostat TX
		TIM3->SR &= ~(TIM_SR_UIF);

#if 0
		stavb = ! stavb;
		if (stavb) {
			BOIL_PIN_LOG1;
		} else {
			BOIL_PIN_LOG0;
		}
		return;
#endif

		//turn of the flag
		switch (OT_thermo_comm.status) {
		case OT_STATUS_WAITING: {
			// timeout reached;
			OT_thermo_comm.status = OT_STATUS_WAIT_CPLT;
			TIM3->DIER &= ~TIM_DIER_UIE;		// disable rx timeout int
			TIM3->CR1  &= ~TIM_CR1_CEN;
			break;
		}
		case OT_STATUS_RX: {
			// timeout of receiving next bit reached;
			manchester_decode_init(&OT_thermo_in);
			//data_comm_index = 0;				//TEST TEST TEST
			TIM3->DIER &= ~TIM_DIER_UIE;		// disable rx timeout int
			break;
		}
		case OT_STATUS_TX_START: {
			// change starting waiting time to standard 500us and continue tx
			TIM3->ARR = TIME_500_US;
			OT_thermo_comm.status = OT_STATUS_TX;
		}
		case OT_STATUS_TX: {
			switch (manchester_encode(&OT_thermo_out)) {

			case MAN_SEND_0: {
				THERMO_PIN_LOG0;
				break;
			}
			case MAN_SEND_1: {
				THERMO_PIN_LOG1;
				break;
			}
			default: {				// end of transmitting
				THERMO_PIN_LOG0;
				//set line to low state
				TIM3->CR1 &= ~TIM_CR1_CEN;
				//end of transmission, turn off the timer
				TIM3->DIER &= ~TIM_DIER_UIE;
				//and turn off the TX interrupt
				OT_thermo_comm.status = OT_STATUS_TX_CPLT;
				break;
			}
			}
		}
		}
	}
	if ((TIM3->SR) & (TIM_SR_CC1IF)) {

		//if capture/compare 1 happened, this is RX from thermostat
		//manchester decoding

		uint16_t capture = TIM3->CCR1;		//save the capture time
		TIM3->DIER &= ~TIM_DIER_UIE;		// disable rx timeout int
		TIM3->EGR |= TIM_EGR_UG;			//reset the timer as soon as possible
		//data_map[data_comm_index++] = capture;  //TEST TEST TEST
		TIM3->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 	//reset CC1 flag
		TIM3->DIER |= TIM_DIER_UIE;			// enable rx timeout int

		if (OT_thermo_comm.status == OT_STATUS_RX) {
			switch (manchester_decode(&OT_thermo_in, capture)) {

			case MAN_RECV_OK: 		 				// OK, wait for next bit
				break;

			case MAN_RECV_FINISHED: {					// OK, finish receiving
				OT_thermo_comm.status = OT_STATUS_RX_CPLT;     // completed
				TIM3->CR1 &= ~TIM_CR1_CEN;	    // timer off
				TIM3->DIER &= ~~(TIM_DIER_CC1IE | TIM_DIER_UIE);  // Cap/comp interrupt disable
				TIM3->CNT = 0;					// zero the counter for next use
				break;
			}

			default: {								// some ERROR, finish receiving
				OT_thermo_comm.status = OT_STATUS_RX_CPLT;     // completed
				OT_thermo_comm.rx_err = 1;      // set error flag
				TIM3->CR1 &= ~TIM_CR1_CEN;	    // timer off
				TIM3->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);  // Cap/comp interrupt disable
				TIM3->CNT = 0;					// zero the counter for next use
				break;
			}
			}
		}
	}
}

/****************************************************************************
	Handles Boiler RX interrupts from TIM1
		- is fired on changing rx line on PA8 pin (capture/compare 1)
		- calls manchester_decode()
*/
void TIM1_CC_IRQHandler(void)
{
	if (((TIM1->SR) & (TIM_SR_CC1IF)) !=0) {
		//if capture/compare 1 happened

		uint16_t capture = TIM1->CCR1;			//save the capture time
		TIM1->DIER &= ~TIM_DIER_UIE;			// disable rx timeout int
		TIM1->EGR |= TIM_EGR_UG;				//reset the timer as soon as possible
		//data_map[data_comm_index++] = capture;  //TEST TEST TEST
		//data_map[data_comm_index++] = GPIOA->IDR & (GPIO_IDR_8 | GPIO_IDR_11);  //TEST TEST TEST
		TIM1->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 	//reset CC1 flag
		TIM1->DIER |= TIM_DIER_UIE;			   // enable rx timeout int

		if (OT_boil_comm.status == OT_STATUS_RX) {
			switch (manchester_decode(&OT_boiler_in, capture)) {

			case MAN_RECV_OK: 		 				// OK, wait for next bit
				return;

			case MAN_RECV_FINISHED: {				// OK, finish receiving
				OT_boil_comm.status = OT_STATUS_RX_CPLT;     // completed

				TIM1->CR1 &= ~TIM_CR1_CEN;		// timer off
				TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);  // Cap/comp interrupt disable
				TIM15->DIER &= ~TIM_DIER_UIE;  	// stop timeout timer
				TIM15->CR1 &= ~TIM_CR1_CEN;    	//
				break;
			}

			default: {								// some ERROR, finish receiving
				OT_boil_comm.status = OT_STATUS_RX_CPLT;     // completed
				OT_boil_comm.rx_err = 1;        // set error flag
				TIM1->CR1 &= ~TIM_CR1_CEN;		// timer off
				TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);  // Cap/comp interrupt disable
				TIM15->DIER &= ~TIM_DIER_UIE;  	// stop timeout timer
				TIM15->CR1 &= ~TIM_CR1_CEN;    	//
				break;
			}
			}
		}
	}
}


/****************************************************************************
	Handles Boiler TX interrupts from TIM1
		- is fired up every 500us
		- calls manchester_encode()
*/

void TIM1_BRK_UP_TRG_COM_IRQHandler(void)
{
	//interrupt for manchester transmit to boiler - PA11 pin
	//logic levels are inverted! log. 0 = 1, log.1 = 0! Macros are defined with this in mind.
	if (((TIM1->SR) & (TIM_SR_UIF)) != 0) { //interrupt caused by update event

		TIM1->SR &= ~TIM_SR_UIF; //cancel the event
		if (OT_boil_comm.status == OT_STATUS_TX) {
			switch (manchester_encode(&OT_boiler_out)) {

				case MAN_SEND_0: {
					BOIL_PIN_LOG0;
					break;
				}
				case MAN_SEND_1: {
					BOIL_PIN_LOG1;
					break;
				}
				default: {							// end of transmitting
					BOIL_PIN_LOG0;
					OT_boil_comm.status = OT_STATUS_TX_CPLT;

					TIM1->CR1 &= ~TIM_CR1_CEN;		//turn off the timer and interrupt
					TIM1->DIER &= ~TIM_DIER_UIE;
				}
			}

		} else if (OT_boil_comm.status == OT_STATUS_RX) {
			// timeout of receiving next bit
			OT_boil_comm.status = OT_STATUS_RX_CPLT;     	// completed
			OT_boil_comm.rx_err = 1;        				// set error flag
			TIM1->CR1 &= ~TIM_CR1_CEN;						// timer off
			TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE); // interrupt off
			TIM15->DIER &= ~TIM_DIER_UIE; 					// interrupt off
			TIM15->CR1 &= ~TIM_CR1_CEN;    					// timer off

		} else if (OT_boil_comm.status == OT_STATUS_RX_START) {
			//data_map[data_comm_index++] = 9999;  //TEST TEST TEST
			OT_boil_comm.status = OT_STATUS_RX;				// start waiting for first bit
			TIM1->SR &= ~TIM_SR_CC1IF; 						// reset possible CC1 flag
			TIM1->DIER |= TIM_DIER_CC1IE; 					// enable CC1 interrupts
			TIM1->DIER &= ~TIM_DIER_UIE; 					// disable timeout interrupts
		}
	}
}


void OT_boiler_TX(void* value)
{

	TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	//interrupts off
	TIM1->CR1  &= ~TIM_CR1_CEN; 					// timer off
	TIM1->ARR = TIME_500_US;
	TIM1->EGR |= TIM_EGR_UG;						//reset the timer
	TIM1->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		// reset CC1 and UIF flag
	BOIL_PIN_LOG0;									// set tx pin to 0

	OT_boil_comm.status = OT_STATUS_TX;

	manchester_encode_init(&OT_boiler_out, value);

	TIM1->DIER |= TIM_DIER_UIE;	//interrupts on
	TIM1->CR1 |= TIM_CR1_CEN;   // timer on
}

void OT_boiler_RX(void)
{

	//TIM1 is set to over 500us, interrupt on Capture on PA8
	//rest of the config is in master_config function

	TIM15->CR1 &= ~TIM_CR1_CEN; 	// turn off TIM15 - timeout couter
	TIM15->DIER &= ~(TIM_DIER_UIE);	// interrupts off
	TIM15->ARR = TIME_LONG_800_MS;	// 800 ms
	TIM15->EGR |= TIM_EGR_UG;		// reset the timer

	TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	//interrupts off
	TIM1->CR1  &= ~TIM_CR1_CEN; 					// timer off
	TIM1->EGR |= TIM_EGR_UG;						//reset the timer
	TIM1->ARR = MAN_BIT_TIME_MAX;				// 1.15 ms - max length of 1 bit
	BOIL_PIN_LOG0;								// set tx pin to 0

	//data_comm_index = 0;		//TEST TEST TEST

	OT_boil_comm.rx_err = 0;
	OT_boil_comm.status = OT_STATUS_RX_START;  // wait one TIME_MAX before reading
	manchester_decode_init(&OT_boiler_in);

	TIM1->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		// reset CC1 and UIF flag
	//TIM1->DIER |= TIM_DIER_CC1IE; //interrupts on
	TIM1->DIER |= TIM_DIER_UIE; 	//interrupts on
	TIM1->CR1 |= TIM_CR1_CEN;     	// timer on

	TIM15->SR &= ~(TIM_SR_UIF); 	// reset UIF flag
	TIM15->DIER |= TIM_DIER_UIE;   	// interrupts on
	TIM15->CR1 |= TIM_CR1_CEN;     	// TIM15 on measuring timeout
}

void OT_boiler_WAIT(uint16_t time)
{

	TIM15->DIER &= ~(TIM_DIER_UIE);			//interrupts off
	TIM15->CR1 &= ~TIM_CR1_CEN; 				// timer off
	TIM15->EGR |= TIM_EGR_UG;				//reset the timer

	BOIL_PIN_LOG0;							// set tx pin to 0
	OT_boil_comm.status = OT_STATUS_WAITING;

	TIM15->ARR = time;				// 100 ms
	TIM15->SR &= ~(TIM_SR_UIF); 	// reset UIF flag
	TIM15->DIER |= TIM_DIER_UIE;	// update interrupt on
	TIM15->CR1 |= TIM_CR1_CEN; 		// timer on
}

void OT_boiler_IDLE(void)
{

	TIM15->CR1 &= ~TIM_CR1_CEN; 					// timer off
	TIM1->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	// interrupts off
	TIM1->CR1  &= ~TIM_CR1_CEN; 					// timer off
	TIM1->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		// reset CC1 and UIF flag
	BOIL_PIN_LOG0;									// set tx pin to 0

	OT_boil_comm.status = OT_STATUS_IDLE;
}


void OT_thermo_RX(void)
{

	TIM3->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	// interrupts off
	TIM3->CR1 &= ~TIM_CR1_CEN; 						// timer off
	TIM3->PSC = TIMER_PRE;
	TIM3->ARR = MAN_BIT_TIME_MAX;					// rx bit timeout
	TIM3->EGR |= TIM_EGR_UG;						//reset the timer
	TIM3->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		// reset CC1 and UIF flag

	//data_comm_index = 0;		//TEST TEST TEST

	//THERMO_PIN_LOG0;									// set tx pin to 0
	OT_thermo_comm.rx_err = 0;
	OT_thermo_comm.boiler_req = 0;
	OT_thermo_comm.status = OT_STATUS_RX;

	manchester_decode_init(&OT_thermo_in);

	TIM3->CNT = 0;	 								// timer counter
	TIM3->DIER |= TIM_DIER_CC1IE; 					// timer interrupt CC on
	TIM3->CR1 |= TIM_CR1_CEN;
}

void OT_thermo_TX(uint32_t* value, uint16_t time)
{

	TIM3->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	//interrupts off
	TIM3->CR1 &= ~TIM_CR1_CEN; 						// timer off
	TIM3->PSC = TIMER_PRE;
	TIM3->EGR |= TIM_EGR_UG;						//reset the timer
	TIM3->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		//reset CC1 and UIF flag
	//THERMO_PIN_LOG0;								// set tx pin to 0

	if (time) {
		// wait for 20ms before sending reply
		TIM3->ARR = time;
		OT_thermo_comm.status = OT_STATUS_TX_START;
	} else {
		TIM3->ARR = TIME_500_US;
		OT_thermo_comm.status = OT_STATUS_TX;
	}
	OT_thermo_comm.boiler_req = 0;  // boiler transaction finished
	manchester_encode_init(&OT_thermo_out, value);

	TIM3->DIER |= TIM_DIER_UIE;		// update interrupt on
	TIM3->CR1 |= TIM_CR1_CEN; 		// timer on
}

void OT_thermo_WAIT(uint16_t time)
{

	TIM3->DIER &= ~(TIM_DIER_CC1IE | TIM_DIER_UIE);	//interrupts off
	TIM3->CR1 &= ~TIM_CR1_CEN; 						// timer off
	THERMO_PIN_LOG0;								// set tx pin to 0

	OT_thermo_comm.status = OT_STATUS_WAITING;

	TIM3->PSC = TIMER_PRE_LONG;
	TIM3->ARR = time;				// 800 ms or other :-)
	TIM3->EGR |= TIM_EGR_UG;						//reset the timer
	TIM3->SR &= ~(TIM_SR_CC1IF | TIM_SR_UIF); 		//reset CC1 and UIF flag
	TIM3->DIER |= TIM_DIER_UIE;		// update interrupt on
	TIM3->CR1 |= TIM_CR1_CEN; 		// timer on
}


void MB_iface_RX(void)
{
	/* stop modbus communication, clear buffers, start receiving */
	DISABLE_USART1_TX;				// transmitter disable
	DISABLE_USART1_RX;				// receiver disable
	Modbus_in_length = 0;
	Modbus_out_index = 0;
	Modbus_out_length = 0;
	Mod_comm.boiler_req = 0;
	Mod_comm.status = MB_STATUS_RX;
	ENABLE_USART1_RX;				// receiver enable
}

void MB_iface_TX(void)
{
	// start transmission  to USART
	//  data must be in Modbus_out[0..Modbus_out_length-1]

	uint16_t mod_crc = Mod_CRC(Modbus_out, Modbus_out_length);	// calc crc
	Modbus_out[Modbus_out_length] = mod_crc & 0xff;				// move crc to buffer
	Modbus_out[Modbus_out_length+1] = mod_crc >> 8;
	Modbus_out_length += 2;										// add +2 to length
	Modbus_out_index = 0;					// prepare counter
	Mod_comm.status = MB_STATUS_TX;
	ENABLE_USART1_TX;						// start hw sendind
	//LED_ON(LED1);
}

void MB_iface_WAIT(uint16_t time)
{
	// set timeout to modbus transaction
	TIM16->DIER &= ~(TIM_DIER_UIE);			//interrupts off
	TIM16->CR1 &= ~TIM_CR1_CEN; 			// timer off
	TIM16->EGR |= TIM_EGR_UG;				//reset the timer

	Mod_comm.status = MB_STATUS_WAITING;

	TIM16->ARR = time;
	TIM16->SR &= ~(TIM_SR_UIF); 	// reset UIF flag
	TIM16->DIER |= TIM_DIER_UIE;	// update interrupt on
	TIM16->CR1 |= TIM_CR1_CEN; 		// timer on
}


inline int is_in_wset( uint8_t k )
{
	// returns read(0)/write(1) status for boiler opentherm comm
	return ((wset[k >> 5] & (1 << (k & 0x1f))) != 0 );
}

void next_datacom_index(void)
{
	// rotate data_id that we ask - at idle
	data_comm_index++;
	if (data_comm_index > 256) {
		data_comm_index = 0;
	}
}

uint16_t timing_read(uint16_t id){
    switch (id){
        case 1000:return BRR_HIGH;          break;
        case 1001:return BRR_LOW;           break;
        case 1002:return TIMER_PRE;         break;
        case 1003:return TIME_500_US;       break;
        case 1004:return TIME_630_US;       break;
        case 1005:return TIME_20_MS;        break;
        case 1006:return TIME_PRE_LONG;     break;
        case 1007:return TIME_LONG_800_MS;  break;
        case 1008:return TIME_LONG_1_MS;    break;
        case 1009:return TIME_LONG_1_S;     break;
    }
}
uint16_t timing_write(uint16_t id, uint16_t data){
        switch (id){
            case 1000:BRR_HIGH = data;          break;
            case 1001:BRR_LOW = data;           break;
            case 1002:TIMER_PRE = data;         break;
            case 1003:TIME_500_US = data;       break;
            case 1004:TIME_630_US = data;       break;
            case 1005:TIME_20_MS = data;        break;
            case 1006:TIME_PRE_LONG = data;     break;
            case 1007:TIME_LONG_800_MS = data;  break;
            case 1008:TIME_LONG_1_MS = data;    break;
            case 1009:TIME_LONG_1_S = data;     break;
        }

        return timing_read(id);
}

void mb_clr_counters(void){
    mb_counters.bus_comm_errors = 0;
    mb_counters.bus_messages = 0;
    mb_counters.char_overrun = 0;
    mb_counters.exception_errors = 0;
    mb_counters.messages = 0;
    mb_counters.nack = 0;
    mb_counters.no_response = 0;
}


void init_ot_msg(OT_msg* msg, uint8_t msgtype, uint8_t dataid, uint16_t value)
{

	msg->msg_type = msgtype;
	msg->data_id = dataid;
	msg->data_value = value;
	if (parity(*(uint32_t*)msg)) {
		msg->msg_type |= 0x80;
	}
}


uint8_t mb_analyze_packet(uint8_t length)
{
    mb_counters.bus_messages++;
    //increment message counter

	if ((length < 3) || (Mod_CRC(Modbus_in, length) != 0)) {
		// frame error or CRC is bad
		mb_counters.bus_comm_errors++;
		//increment bad message counter
		return(0);
	}

	if ((mb_address(Modbus_in) != modbus_cfg.address) && (mb_address(Modbus_in) != 0)) {
		// is modbus is not our nor broadcast?
		return(0);
	}

	switch (mb_opcode(Modbus_in)) { // so, what modbus function we have to do?

    mb_counters.messages++;
    //increment message_for_us counter

	case MB_OP_READ_HOLD_REG:
    case MB_OP_DIAGNOSTIC:{//Diag functions
        if (mb_regaddr(Modbus_in) == MB_DIAG_RETURN_QUERY){
            mb_writecoil_response();  //return query, same as coil response
            return(1);
        }
        if (mb_regaddr(Modbus_in) == MB_DIAG_CLEAR_COUNTERS_AND_DIAG){
            mb_clr_counters();
            mb_writecoil_response();  //clear counters, echo reply
            return(1)
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_BUS_MSG_CNT){
            mb_set_reg_response(mb_counters.bus_messages);//return bus message count
            return(1);
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_BUS_COM_ERR_CNT){
            mb_set_reg_response(mb_counters.bus_comm_errors);
            return(1);
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_BUS_EXC_ERR_CNT){
            mb_set_reg_response(mb_counters.exception_errors);
            return(1);
        }

        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_SLAVE_MSG_CNT){
            mb_set_reg_response(mb_counters.messages);
            return(1);
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_SLAVE_NORES_CNT){
            mb_set_reg_response(mb_counters.no_response);
            return(1);
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_SLAVE_NAK_CNT){
            mb_set_reg_response(mb_counters.nack);
            return(1);
        }
        if (mb_regaddr(Modbus_in) = MB_DIAG_RET_BUS_CHAR_OVRN_CNT){
            mb_set_reg_response(mb_counters.char_overrun);
            return(1);
        }

        // others not implemented

        mb_set_error(MB_ERR_ILLEGAL_FUNCTION);
        return(1);



    }
	case MB_OP_READ_REG: { // Read input register(s)
		if ((mb_regaddr(Modbus_in) > 256) && (mb_regaddr(Modbus_in) < 1000)) {
			// Bad register address
			mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
			return(1);
		}

		if ((mb_regaddr(Modbus_in) >= 1000) && (mb_regaddr(Modbus_in) <= 1009)){
            // Internal timing settings
            mb_set_reg_response(timing_read(mb_regaddr(Modbus_in)));
            return (1);
		}
		if ((mb_regaddr(Modbus_in) + mb_regcount(Modbus_in) > 257) || (mb_regcount(Modbus_in) > 254)) {
			// Bad register count
			mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
			return(1);
		}
		if ((mb_regcount(Modbus_in) > 1) || (main_mode != CONTROLLER) || (mb_opcode(Modbus_in)==MB_OP_READ_HOLD_REG)) {
			// Offline response
			mb_set_big_response(data_map);
			return(1);
		}
		//only one register to read - prepare Opentherm message
		init_ot_msg(&OT_boiler_mb_data, OT_MSGTYPE_READ, mb_regaddr(Modbus_in), 0);
		return(2);
	}

	case MB_OP_WRITE_REGS: {  // write multi register -we support only one
		if ((mb_regaddr(Modbus_in) > 255) && (mb_regaddr(Modbus_in) < 1000) ||(mb_regcount(Modbus_in) > 1)) {
			// Bad register address
			mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
			return(1);
		}
		if (main_mode != CONTROLLER) {
			mb_set_error(MB_ERR_SLAVE_DEVICE_BUSY);
			return(1);
		}

		if ((mb_regaddr(Modbus_in) >= 1000) && (mb_regaddr(Modbus_in) <= 1009)){
            // Internal timing settings
            mb_set_reg_response(timing_write(mb_regaddr(Modbus_in),mb_value(Modbus_in)));
            return (1);
		}

		//write one register - direct to Opentherm
		init_ot_msg(&OT_boiler_mb_data, OT_MSGTYPE_WRITE, mb_regaddr(Modbus_in), mb_mvalue(Modbus_in));
		if ((OT_boiler_mb_data.data_id <= 255)) {
			// if mb message is write_data and data_id is valid
			data_map[OT_boiler_mb_data.data_id] = OT_boiler_mb_data.data_value;
		}
		return(2);
	}

	case MB_OP_WRITE_REG: {  // write single register
		if ((mb_regaddr(Modbus_in) > 255) && (mb_regaddr(Modbus_in) < 1000)) {
			// Bad register address
			mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
			return(1);
		}
		if (main_mode != CONTROLLER) {
			mb_set_error( MB_ERR_SLAVE_DEVICE_BUSY);
			return(1);
		}

		if ((mb_regaddr(Modbus_in) >= 1000) && (mb_regaddr(Modbus_in) <= 1009)){
            // Internal timing settings
            mb_set_reg_response(timing_write(mb_regaddr(Modbus_in),mb_value(Modbus_in)));
            return (1);
		}
		//write one register - direct to Opentherm
		init_ot_msg(&OT_boiler_mb_data, OT_MSGTYPE_WRITE, mb_regaddr(Modbus_in), mb_value(Modbus_in));
		if ((OT_boiler_mb_data.data_id <= 255)) {
			// if mb message is write_data and data_id is valid
			data_map[OT_boiler_mb_data.data_id] = OT_boiler_mb_data.data_value;
		}
		return(2);
	}

	case MB_OP_WRITE_COIL: { // write single coil - mode switch
		if (mb_regcount(Modbus_in) == 0xff00) {
			if (mb_regaddr(Modbus_in) == 0) {
				main_mode = CONTROLLER;
			}
			if (mb_regaddr(Modbus_in) == 1) {
				main_mode = PASSIVE_MONITOR;
			}
			if (mb_regaddr(Modbus_in) > 2){
                mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
                return(1);
			}
		} else {
            main_mode = MONITOR;
		}
		mb_writecoil_response();
        return(1);


	}

	case MB_OP_READ_BIT:      // read discrete input(s)
	case MB_OP_READ_COIL: {   // read coil(s)
		if ((mb_regcount(Modbus_in) == 1) && (mb_regaddr(Modbus_in) == 0)) {
			mb_set_coil_response(main_mode & 1);
			return(1);
		}
		mb_set_error(MB_ERR_ILLEGAL_DATA_ADDRESS);
		return(1);
	}

	default: { //any other message not implemented
		mb_set_error(MB_ERR_ILLEGAL_FUNCTION);
		return(1);
	}
	} /* switch */
}


void master_init(void)
{

	SystemCoreClockUpdate(); // set global variable SystemCoreClock

	//DATA WRITE SET INIT
	// at idle, we have to send write on Opentherm in following data ids:
	// 1, 2, 4, 7, 8, 14, 16, 23, 24 	= 0x1814196 on wset[0]
	// 124, 126 						= 0x50000000 on wset[3]
	wset[0] = 0x1814196;
	wset[1] = 0;
	wset[2] = 0;
	wset[3] = 0x50000000;

	RCC->AHBENR |= RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN;
	//clock on to A, B and C ports

	RCC->APB2ENR |= RCC_APB2ENR_TIM1EN | RCC_APB2ENR_TIM15EN | RCC_APB2ENR_TIM16EN | RCC_APB2ENR_USART1EN | RCC_APB2ENR_SYSCFGCOMPEN;
	//clock to timer 1, timer 15, USART1 and comparator

	RCC->APB1ENR |= RCC_APB1ENR_DACEN | RCC_APB1ENR_TIM2EN | RCC_APB1ENR_TIM3EN ;
	//clock to DAC and Timer 3

	GPIOB->PUPDR = GPIO_PUPDR_PUPDR0_0 | GPIO_PUPDR_PUPDR1_0 | GPIO_PUPDR_PUPDR3_0 | GPIO_PUPDR_PUPDR4_0 | GPIO_PUPDR_PUPDR5_0;
	GPIOA->PUPDR = GPIO_PUPDR_PUPDR8_0 | GPIO_PUPDR_PUPDR13_0 | GPIO_PUPDR_PUPDR14_0 | GPIO_PUPDR_PUPDR15_0;
	//PB0, 1, 3, 4, 5 = pull-up on
	//PA8, 13, 14, 15 = pull-up on

	GPIOA->MODER |= GPIO_MODER_MODER6_0 | GPIO_MODER_MODER7_0;
	//PA6, 7 is OUT - LED1, LED2
	/*
	// LEDs on development board
	GPIOC->MODER |= GPIO_MODER_MODER8_0 | GPIO_MODER_MODER9_0; //test led
	GPIOC->OTYPER &= ~(GPIO_OTYPER_OT_8 | ~GPIO_OTYPER_OT_9);
	GPIOC->OSPEEDR |= (GPIO_OSPEEDER_OSPEEDR8|GPIO_OSPEEDER_OSPEEDR9);
	GPIOC->PUPDR &= ~(GPIO_PUPDR_PUPDR8|GPIO_PUPDR_PUPDR9);
	*/

	GPIOA->MODER |= GPIO_MODER_MODER11_0;
	//PA11 is OUT - OT boiler TX pin
	GPIOA->MODER |= GPIO_MODER_MODER8_1;
	GPIOA->AFR[1] |= 0x00000002;
	//PA8  is ALT - alternate function 2 = TIM1 CH1   boiler RX

	GPIOA->MODER |= GPIO_MODER_MODER1 | GPIO_MODER_MODER4;
	//PA1 - analog mode - thermostat RX
	//PA4 - analog mode - thermostat TX


	GPIOA->MODER |= GPIO_MODER_MODER9_1 | GPIO_MODER_MODER10_1 | GPIO_MODER_MODER12_1;
	//PA9, 10, 12 = alternate function mode

	GPIOA->AFR[1] |= ((0x00000001) << 4) | ((0x00000001) << 8) | ((0x00000001) << 16) ;
	//PA9 = alternate function 1 = USART1 TX
	//PA10 = alternate function 1 = USART1 RX
	//PA12 = alternate function 1 = USART1 RTS



	//USART cfg read
	//--------------
	// !! zmeneno posunuto
	//ADDRESS: 	6 bit (MSB first) 	PB5, PB4, PA13, PA14, PA15, PB3
	//BAUD:     1 bit				PB1								on = 9600 (low)	off = 19200 (high)
	//PARITY: 	1 bit 				PB0 							on = ODD		off = EVEN

	modbus_cfg.address = 0x0;
	if ((GPIOA->IDR & GPIO_IDR_13) == 0 ) {
		modbus_cfg.address |= 0x20;
	}
	if ((GPIOA->IDR & GPIO_IDR_14) == 0 ) {
		modbus_cfg.address |= 0x10;
	}
	if ((GPIOA->IDR & GPIO_IDR_15) == 0 ) {
		modbus_cfg.address |= 0x08;
	}
	if ((GPIOB->IDR & GPIO_IDR_3 ) == 0 ) {
		modbus_cfg.address |= 0x04;
	}
	if ((GPIOB->IDR & GPIO_IDR_4 ) == 0 ) {
		modbus_cfg.address |= 0x02;
	}
	if ((GPIOB->IDR & GPIO_IDR_5 ) == 0 ) {
		modbus_cfg.address |= 0x01;
	}

	//USART_cfg.address = 1; //temporary hack

	modbus_cfg.parity = ((GPIOB->IDR & GPIO_IDR_1) == 0);
	modbus_cfg.baud_low = ((GPIOB->IDR & GPIO_IDR_0) == 0);

	//USART1 config
	//-----------------------------------
	USART1->CR1 |= USART_CR1_PCE;
	//Parity control enable

	USART1->CR1 |= USART_CR1_M0;
	//With PCE=1 this means 8 bit and parity

	if (modbus_cfg.parity) {
		USART1->CR1 |= USART_CR1_PS;
	} else {
		USART1->CR1 &= ~USART_CR1_PS;
	}
	//parity set

	if (modbus_cfg.baud_low) { // baud rate set
		USART1->BRR = BRR_LOW;
	} else {
		USART1->BRR = BRR_HIGH;
	}

	//Driver Enable mode  enable - RS485
	USART1->CR3 |= USART_CR3_DEM;
	USART1->CR1 |= USART_CR1_DEAT_3;  // Assertation time; in 1/16 bit time. DEAT_3 means 1/2 bit
	USART1->CR1 |= USART_CR1_DEDT_3;  // Deass time
	//USART1->CR3 |= USART_CR3_DEP; // polarity of DE

	// Modbus RTU
	USART1->CR2 |= USART_CR2_RTOEN;	//receiver timeout enable
	USART1->RTOR = 22;              //Receiver timeout - 22 bit = 2*(1+8+1+1)

	NVIC_EnableIRQ(USART1_IRQn);    // USART1 interrupt enable
	NVIC_SetPriority(USART1_IRQn,0);

	USART1->CR1 |= USART_CR1_UE;	// USART enable
	DISABLE_USART1_RX;				// receiver disable
	DISABLE_USART1_TX;				// transmitter disable


	//---------------------------------------------------
	//Comparator 1

	COMP1->CSR |= COMP_CSR_COMP1OUTSEL_1 | COMP_CSR_COMP1OUTSEL_2;
	// Comp 1 output = Timer 3 input capture 1

	COMP1->CSR |= COMP_CSR_COMP1INSEL_0 | COMP_CSR_COMP1INSEL_1;
	// Comp 1 inverting input = Vrefint

	COMP1->CSR |= COMP_CSR_COMP1HYST_0;
	// Comp 1 Low hysteresis

	COMP1->CSR |= COMP_CSR_COMP1EN;
	// Comparator 1 enable

	//---------------------------------------------------
	//DAC1

	DAC->CR |= DAC_CR_EN1;
	//DAC enable

	THERMO_PIN_LOG0;
	//set Log. 0 on DAC output

	//---------------------------------------------------
	//Timer 1 = main timer for boiler Opentherm comm
	//used in RX - update interrupt and TX - CC interrupt

	TIM1->CCMR1 |= TIM_CCMR1_CC1S_0 ;//| TIM_CCMR1_IC1F_1 | TIM_CCMR1_IC1F_0 | TIM_CCMR1_IC1F_2 | TIM_CCMR1_IC1F_3;
	//CC channel 1 as input, IC1 mapped on TI1
	//set digital filter on TI1 8x f.INT

	TIM1->CCER |= TIM_CCER_CC1NP | TIM_CCER_CC1P | TIM_CCER_CC1E;
	//Cap/comp 1 enable, both edges

	TIM1->PSC = TIMER_PRE;
	//Prescaler set (typical 1us)

	//TIM1->DIER |= TIM_DIER_CC1IE;
	// Timer 1 Cap/Comp interrupt enable

	//TIM1->DIER |=TIM_DIER_UIE;
	// Timer 1 update interrupt enable

	NVIC_EnableIRQ(TIM1_CC_IRQn); 					//turn the timer CC interupt on
	NVIC_SetPriority(TIM1_CC_IRQn, 1);
	NVIC_EnableIRQ(TIM1_BRK_UP_TRG_COM_IRQn); 		//turn the timer update interrupt on
	NVIC_SetPriority(TIM1_BRK_UP_TRG_COM_IRQn, 1);

	//---------------------------------------------------
	//Timer 15 - aux timer for timeouts in boiler Opentherm comm

	TIM15->PSC = TIMER_PRE_LONG;
	//Prescaler set (typical 1ms)

	//TIM15->CR1 |= TIM_CR1_OPM;
	// Timer 15 one pulse mode enable - not needed

	//TIM15->DIER |= TIM_DIER_UIE;
	//Timer 15 update interrupt enable

	NVIC_EnableIRQ(TIM15_IRQn);
	NVIC_SetPriority(TIM15_IRQn,2);
	//Timer 15 interrupt on and priority set

	//---------------------------------------------------
	//Timer 16 - aux timer for timeouts in Modbus comm

	TIM16->PSC = TIMER_PRE_LONG;
	//Prescaler set (typical 1ms)

	NVIC_EnableIRQ(TIM16_IRQn);
	NVIC_SetPriority(TIM16_IRQn,2);
	//Timer 16 interrupt on and priority set

	//---------------------------------------------------
	//Timer 3 - main timer for thermostat Opentherm comm

	TIM3->CCMR1 |= TIM_CCMR1_CC1S_0;
	// Cap/comp as input, IC1 mapped on TI1

	TIM3->CCER |= TIM_CCER_CC1NP | TIM_CCER_CC1P | TIM_CCER_CC1E;
	// Cap/com 1 enable, both edges

	//TIM3->DIER |= TIM_DIER_CC1IE;
	// Cap/com 1 interrupt enable

	//TIM3->DIER |= TIM_DIER_UIE;
	// Update interrupt enable -

	NVIC_EnableIRQ(TIM3_IRQn);
	NVIC_SetPriority(TIM3_IRQn, 1);
	//Interrupt  vector enable and priority set
}

#if 0
void testing_loop(void)
{
	// simple loop for debugging USART communication - wait for data, reply with OK
	while (1) {

		if (Mod_comm.mod_rx_cplt) { 			// modbus packet received
			//LED_ON(LED1);
			if (!Mod_comm.mod_rx_error) {							// packet is OK

				//--------------------------------------------
				Modbus_out_length = 4;
				Modbus_out[0] = 'O';
				Modbus_out[1] = 'K';
				Modbus_out[2] = 13;
				Modbus_out[3] = 10;
				//--------------------------------------------

				Modbus_out_index = 0;			// start transmission
				Mod_comm.mod_tx_acty = 1;
				ENABLE_USART1_TX;

			} else { 							// packet is BAD, dont reply
				MB_iface_RX();					// restart modbus session
			}
			//Mod_comm.mod_rx_cplt = 0;			// clear the rx_cplt flag, rx_acty flag stays up
		}

		if (Mod_comm.mod_tx_cplt) { 			// modbus reply packet was sent
			MB_iface_RX();						// restart modbus session
		}
	}
}
#endif

int main(void)
{
	//setup_test();
	master_init();
	main_mode = MONITOR;
	//main_mode = CONTROLLER;
	MB_iface_RX();						// reset Modbus interface

#if 0
	THERMO_PIN_LOG0;				// reset boiler tx pin
	BOIL_PIN_LOG0;		  		    // reset boiler tx pin
	OT_boiler_RX();				    // start thermo rx mode
	OT_thermo_TX((uint32_t*)&OT_thermo_in_data,0);				    // start thermo rx mode
	while (data_comm_index < 100) {}
#endif

	THERMO_PIN_LOG0;				// reset boiler tx pin
	BOIL_PIN_LOG0;		  		    // reset boiler tx pin
	OT_boiler_IDLE();			    // start boiler idle mode
	OT_thermo_RX();				    // start thermo rx mode
	//testing_loop();
	while(1) {
		/*
		 * ------------MAIN LOOP-----------------
		 *
		 * Checking status of all three interfaces - Modbus, OT Boiler, OT Thermostat
		 *
		 * Modbus:
		 * 		- in idle time is listening for packet (RX). New packet is signaled by MB_STATUS_RX_CPLT
		 * 		- packet is analyzed and interface can change status to
		 * 				- MB_STATUS_WAITING if request requires reply from boiler, or
		 * 				- MB_STATUS_TX if request can be answered immediatelly or in case of error
		 * 		- if remote request is resolved before timeout, modbus interface goes to TX mode by sending reply or error
		 * 		- if timeout in waiting mod is done, modbus interface returns to RX
		 * 		- after successful sending, interface goes to RX
		 *
		 * OT Boiler:
		 * 		- in IDLE state is checked if there is some remote request from modbus
		 * 		  periodically generates scan requests of boiler
		 * 		- in case of new request interface goes to TX mode
		 * 		- after sending (TX_CPLT) waits for 1ms and changes mode to RX and waits for reply.
		 * 		  At the same time is starts timeout timer (800ms)
		 * 		- in case of successful receiving, reply is saved or sent to original requestor. Interface  goes to WAITING state
		 * 		  to satisfy 100ms gap to next transaction
		 * 		- in case of timeout, error reply can be sent to original requestor and intrface goes to IDLE state
		 *
		 * OT Thermostat:
		 * 		- in idle time is listening for OT frame. New packet is signaled by OT_STATUS_RX_CPLT
		 * 		- if program is in MONITOR mode, packet is resent to OT boiler
		 * 		- in CONTROLLER mode is generated reply from memory data_map
		 *
		 * 		-
		 *
		*/

		switch (Mod_comm.status) {

		case MB_STATUS_RX_CPLT: {			// MODBUS received new message

			switch (mb_analyze_packet(Modbus_in_length)) {
			case 0: { 					// error or packet is not for us
				MB_iface_RX();
				break;
			}
			case 1: {					// reply is prepared
				//LED_ON(LED1);
				MB_iface_TX();
				break;
			}
			case 2: {
				//LED_ON(LED1);			// required remote boiler request for reply
				Mod_comm.boiler_req = 1;
				MB_iface_WAIT(TIME_LONG_1_S);
				break;
			}
			}
			break;
		}

		case MB_STATUS_TX_CPLT: {			// MODBUS reply packet was sent
			//LED_OFF(LED1);
			MB_iface_RX();					// restart modbus session
			break;
		}

		case MB_STATUS_WAIT_CPLT: {			// MODBUS transaction timeout
			//error mb_counter
			//LED_OFF(LED1);
			MB_iface_RX();					// restart modbus session
			break;
		}
		}

		switch (OT_thermo_comm.status) {

		case OT_STATUS_RX_CPLT: {			// IF_THERMO received new request

			if (!OT_thermo_comm.rx_err && (parity(*((uint32_t*)&OT_thermo_in_data)) == 0)) {
				LED_ON(LED1);
				// OT packet is OK
				if (main_mode == MONITOR) {     		// schedule request to boiler
					OT_thermo_comm.boiler_req = 1;  	// boiler transaction required
					OT_thermo_WAIT(TIME_LONG_800_MS);	// start waiting mode max 800ms
					//OT_thermo_WAIT(0xffff);	// start waiting mode max 800ms
					if (((OT_thermo_in_data.msg_type & OT_MSGTYPE_MASK) == OT_MSGTYPE_WRITE) &&
					    (OT_thermo_in_data.data_id <= 127)) {
						// if thermostat message is write_data and data_id is valid
						data_map[OT_thermo_in_data.data_id] = OT_thermo_in_data.data_value;
						//update data_map with it
					}
				} //else {						// reply to thermo from memory, with gap 20ms - collisions, collisions! Not good!
				  //	
				  //	uint8_t idx = OT_thermo_in_data.data_id;
				  //	if (idx > 255) {
				  //		init_ot_msg(&OT_thermo_in_data, OT_MSGTYPE_UNKNOWN_DATAID, idx, 0);
				  //	} else {
				  //		init_ot_msg(&OT_thermo_in_data, ((OT_thermo_in_data.msg_type & OT_MSGTYPE_MASK)==OT_MSGTYPE_WRITE) ? OT_MSGTYPE_WRITE_ACK : OT_MSGTYPE_READ_ACK,
				 //		            idx, data_map[idx]);
				 //	}
				 //	OT_thermo_TX((uint32_t*)&OT_thermo_in_data, TIME_20_MS);
				}

			} else {
				OT_thermo_WAIT(TIME_LONG_1_MS);			// start waiting for recovery from error
			}
			break;
		}

		case OT_STATUS_TX_CPLT: {			// IF_THERMO sent reply
			LED_OFF(LED1);
			data_map[96] = TIM3->CNT;
			OT_thermo_WAIT(TIME_LONG_1_MS);				// start waiting mode for 1 ms
			break;
		}

		case OT_STATUS_WAIT_CPLT: {			// IF_THERMO timeout reached
			//data_map[95] = TIM15->CNT;
			OT_thermo_RX();				    // start rx mode
			break;
		}
		}

		switch (OT_boil_comm.status) {

		case OT_STATUS_IDLE: {				// IF_BOILER: can send something
			if (OT_thermo_comm.boiler_req) {
				OT_boil_comm.requestor = OT_REQ_THERMO;
				OT_boiler_TX(&OT_thermo_in_data); // data are from thermostat

			} else if (Mod_comm.boiler_req) {

				OT_boil_comm.requestor = OT_REQ_MODBUS;
				OT_boiler_TX(&OT_boiler_mb_data); // data prepared from modbus

			} else if ((main_mode == CONTROLLER)) {
				if (is_in_wset(data_comm_index)) {
					init_ot_msg(&OT_boiler_mb_data, OT_MSGTYPE_WRITE, data_comm_index, data_map[data_comm_index]);
				} else {
					init_ot_msg(&OT_boiler_mb_data, OT_MSGTYPE_READ, data_comm_index, 0);
				}
				OT_boil_comm.requestor = OT_REQ_CYCLING;
				OT_boiler_TX(&OT_boiler_mb_data);
			}
			break;
		}

		case OT_STATUS_TX_CPLT: {			// IF_BOILER: request was sent
			data_map[97] = TIM3->CNT;
			LED_ON(LED2);
			OT_boiler_RX();					// wait for reply from boiler
			break;
		}

		case OT_STATUS_RX_CPLT: {			// IF_BOILER: received reply

			if ((parity(OT_boiler_in.data) == 0) && (!OT_boil_comm.rx_err)) {
				data_map[98] = TIM15->CNT;
				data_map[99] = TIM3->CNT;
				LED_OFF(LED2);
				if ((OT_boil_comm.requestor == OT_REQ_MODBUS) && Mod_comm.boiler_req
				    && (Mod_comm.status == MB_STATUS_WAITING)) {

					Mod_comm.boiler_req = 0;
					switch (OT_boiler_in_data.msg_type & OT_MSGTYPE_MASK) { // what type of message we gathered:

					case OT_MSGTYPE_READ_ACK: {   // we construct the modbus response
						mb_set_reg_response(OT_boiler_in_data.data_value);
						break;
					}

					case OT_MSGTYPE_WRITE_ACK: {  //response is the same as request
						mb_set_response(6);
						break;
					}

					default: {   				 //something went wrong
						mb_set_error(MB_ERR_SLAVE_DEVICE_FAILURE);
						break;
					}
					}
					if (mb_address(Modbus_in) != 0) {
						MB_iface_TX();				// start mb transmission
					}

				} else if ((OT_boil_comm.requestor == OT_REQ_THERMO) && OT_thermo_comm.boiler_req
				           && (OT_thermo_comm.status == OT_STATUS_WAITING)) {
					// send reply to thermostat
					OT_thermo_TX((uint32_t*)&OT_boiler_in_data, 0);
				}
				if (((OT_boiler_in_data.msg_type & OT_MSGTYPE_MASK) == OT_MSGTYPE_READ_ACK) &&
				    (OT_boiler_in_data.data_id <= 255)) {  // if data_id is valid
					// update data_map with it
					data_map[OT_boiler_in_data.data_id] = OT_boiler_in_data.data_value;
				}
				if (OT_boil_comm.requestor == OT_REQ_CYCLING) {
					next_datacom_index();
				}

			} else {
				// some error on boiler
				if ((OT_boil_comm.requestor == OT_REQ_MODBUS) && Mod_comm.boiler_req
				    && (Mod_comm.status == MB_STATUS_WAITING)) {
					// send error to modbus
					Mod_comm.boiler_req = 0;
					if (mb_address(Modbus_in)!=0) {
						mb_set_error(MB_ERR_SLAVE_DEVICE_FAILURE);
						MB_iface_TX();				// start mb transmission
					}
				}
				// don't reply to Thermo on error, transaction stops on timeout
			}
			OT_boiler_WAIT(100);			// minimal time to next OT message
			break;
		}

		case OT_STATUS_RX_TIME_CPLT: {		// IF_BOILER: timeout on RX
			if ((OT_boil_comm.requestor == OT_REQ_MODBUS) && Mod_comm.boiler_req
			    && (Mod_comm.status == MB_STATUS_WAITING)) {
				// send error to modbus
				Mod_comm.boiler_req = 0;
				if (mb_address(Modbus_in)!=0) {
					mb_set_error(MB_ERR_SLAVE_DEVICE_FAILURE);
					MB_iface_TX();				// start mb transmission
				}
			}
			OT_boiler_IDLE();
			break;
		}

		case OT_STATUS_WAIT_CPLT: {			// IF_BOILER: timeout reached
			OT_boiler_IDLE();
			break;
		}
		}
	}
}
