/****************************************************************

	Member of opentherm - Modbus - RPi gateway project
	(c) Vclav Steiger, 2015  Faster CZ, spol. s r.o.

****************************************************************/


#include "manchester.h"

__inline__ void manchester_encode_init(volatile MAN_TX_data* TX, void* data) 
{
	TX->data = *(uint32_t*) data; // copy data
	TX->middle = 0;
	TX->cnt = 0;
}

int manchester_encode(volatile MAN_TX_data* TX)
{
	int result = MAN_SEND_FINISHED;

	switch (TX->cnt){

		case 0:                       // transmit start bit
				if (TX->middle == 0){ // first edge is log.1

					result = MAN_SEND_1;

				} else {              // continues log. 0, and start bit is complete

					result = MAN_SEND_0;
					TX->cnt = 1;
				}
				break;

		case 33: // if cnt = 33, we have to transmit stop bit

				if (TX->middle == 0){

					result = MAN_SEND_1;

				} else {

					result = MAN_SEND_FINISHED; // finish transmitting
					TX->cnt = 34;
				}
				break;

		default: // data transmission

				if (TX->middle == 0){ // if we are at the beginning of the bit
					if (((*(uint32_t*)&TX->data) & (1 << (32-TX->cnt))) !=0) {
						// MSB first, so for cnt=1 if 31.bit is 1

						result = MAN_SEND_1;

					} else {

						result = MAN_SEND_0;
					}

				} else { // we are in the middle of the bit
					if (((*(uint32_t*)&TX->data) & (1 << (32-TX->cnt))) !=0) {
						// MSB first, so for cnt=1 if 31.bit is 1

						result = MAN_SEND_0;
						TX->cnt++;

					} else {

						result = MAN_SEND_1;
						TX->cnt++;
					}
				}
				break;
		}

	TX->middle ^= 1;  // toggle middle
	return result;
}

__inline__ void manchester_decode_init(volatile MAN_RX_data* RX) 
{
	RX->data = 0; // zero the data
	RX->cnt = 0;
}


int manchester_decode(volatile MAN_RX_data* OT, uint16_t capture)
{
	if (OT->cnt == 0) {
			/* This is the first raising edge on line. Capture time is almost without sense. 
			    */
			OT->prev_capture = 0;
			OT->cnt++;
			return MAN_RECV_OK;
	}

	/* check timing 
		- impuls must be at least 250 us long (MAN_ERR)
		- bit time (short+short or long) must be in interval <900us, 1150us> */
	if ((capture < MAN_ERR) ||
        ((OT->prev_capture + capture < MAN_BIT_TIME_MIN) && (OT->prev_capture != 0)) ||
        (OT->prev_capture + capture > MAN_BIT_TIME_MAX)) { 
				/* This is bad interval between edges. Cancel receiving.  */
				return MAN_RECV_ERROR;
	}

	switch (OT->cnt) {

		case 1:{
				/* this is start bit. Capture time shall be short. */
				OT->bit = 1;  // start bit is ever 1
				OT->prev_capture = 0;
				OT->cnt++;
				return MAN_RECV_OK;
			}

		case 34:{	// cnt = 34, end of reception, leaving stop bit
				    // there can be yet another edge. depends on last bit - ignoring
				return MAN_RECV_FINISHED; // RX is complete
			}

		default:{						
				// data reception
				if (capture > MAN_TRESHOLD){ 		 // if capture is long

					OT->bit ^= 1; 						 // toggle the bit (Manchester)
					*(uint32_t*)&OT->data <<= 1;      // shift register
					*(uint32_t*)&OT->data |= OT->bit; // write bit
				    OT->prev_capture = 0;
					OT->cnt++;

				} else if (OT->prev_capture != 0) {      // capture is short-short = we are on bit border
                    									 // received bit is the same as previous 
					*(uint32_t*)&OT->data <<= 1;      // shift register; MSB goes first
					*(uint32_t*)&OT->data |= OT->bit; // write bit
				    OT->prev_capture = 0;
					OT->cnt++;

				} else { 								// we are inside  bit interval, save capture
				    OT->prev_capture = capture;
				}
			break;
			}
	} // switch
	return MAN_RECV_OK;
}

