/****************************************************************

	Member of opentherm - Modbus - RPi gateway project
	(c) Vclav Steiger 2015  Faster CZ, spol. s r.o.

****************************************************************/


#include "modbus.h"

extern volatile uint8_t Modbus_in[255];
extern volatile uint8_t Modbus_out[255];
extern volatile uint8_t Modbus_out_length;
//extern struct struct_modbus_cfg modbus_cfg;
//extern uint8_t main_mode;

/* computes even parity */

int parity(uint32_t d)
{
	uint32_t p;
	p = d ^ (d >> 1);
	p = p ^ (p >> 2);
	p = p ^ (p >> 4);
	p = p ^ (p >> 8);
	p = p ^ (p >>16);
	return p & 1;
}


#define	CRC_POLY 0xA001		// CRC polynomial


uint16_t CRC_step(volatile uint8_t msg, uint16_t mod_crc){
	// one step in computing CRC for Modbus
	mod_crc ^= msg;
	uint8_t i;
	for (i = 0; i<8; i++){
		if (((mod_crc) & 1) == 1) {
			mod_crc = mod_crc >> 1;
			mod_crc ^= CRC_POLY;
		} else {
			mod_crc = mod_crc >> 1;
		}
	}
	return mod_crc;
}

uint16_t Mod_CRC(volatile uint8_t* data, uint8_t length)
{
	// compute CRC - result is not swapped
	// or check message with crc appended - result must be 0 if crc is OK

	uint16_t mod_crc = 0xFFFF;
	uint8_t j;

	for (j = 0; j < length; j++) {

		mod_crc = CRC_step(data[j], mod_crc);
	}
	// mod_crc = swap_word(mod_crc);  //CRC is LSbyte first, need to swap!
	// not needed now
	return mod_crc;
}


void mb_set_response(uint8_t length)
{
	uint8_t k;

	for (k = 0; k < length; k++) {
		Modbus_out[k] = Modbus_in[k];
	}
	Modbus_out_length = length;
}


void mb_set_reg_response(uint16_t value)
{// single register
	Modbus_out[0] = mb_address(Modbus_in);
	Modbus_out[1] = mb_opcode(Modbus_in);
	Modbus_out[2] = 2; // byte count
	Modbus_out[3] = (value >> 8); //data Hi
	Modbus_out[4] = (value & 255);//data Lo
	Modbus_out_length = 5;
}

void mb_set_coil_response(uint8_t mask)
{ //single coil
	Modbus_out[0] = mb_address(Modbus_in);
	Modbus_out[1] = mb_opcode(Modbus_in);
	Modbus_out[2] = 1;
	Modbus_out[3] = mask;
	Modbus_out_length = 4;

}

void mb_writecoil_response(void)
{
	uint8_t k;

	for (k = 0; k < 6; k++) {
		Modbus_out[k] = Modbus_in[k];
	}
	Modbus_out_length = 6;
}


void mb_set_big_response(uint16_t* data_map)
{
	//multiple registers - cached data 
	uint8_t k = mb_regaddr(Modbus_in);
	uint8_t length = mb_regcount(Modbus_in);

	Modbus_out[0] = mb_address(Modbus_in);
	Modbus_out[1] = mb_opcode(Modbus_in);
	Modbus_out[2] = length*2;		// byte count = 2*data req'd qty
	Modbus_out_length = 3;

	for (; length > 0; length--){ // get data from map

		Modbus_out[Modbus_out_length++] = (data_map[k] >> 8); // and put them
		Modbus_out[Modbus_out_length++] = (data_map[k++] & 255);// to the TX message
	}
	//Modbus_out_length = 3 + length*2;
}


void mb_set_error(uint8_t errorcode)
{
	// we respond with message error
	mb_counters.exception_errors++;
	Modbus_out[0] = mb_address(Modbus_in);
	Modbus_out[1] = mb_opcode(Modbus_in) | 0x80; // error function code
	Modbus_out[2] = errorcode; 					 // exception code
	Modbus_out_length = 3;
}



